#ifndef __SBDAQ__
#define __SBDAQ__

#define SB_TIMEOUT			10000				// 10 sec time out
#define SB_MONO				1					// mono recording
#define SB_STEREO			2					// stereo recording
#define SB_CH_LEFT			0
#define SB_CH_RIGHT			1
#define SB_RATE_8KHZ		8000				//  8 kHz sampling rate
#define SB_RATE_11KHZ  		11025				// 11 kHz sampling rate
#define SB_RATE_22KHZ  		22050				// 22 kHz sampling rate
#define SB_RATE_44KHZ  		44100				// 44 kHz sampling rate
#define SB_8BIT				8					//  8 bits resolution
#define SB_16BIT			16					// 16 bits resolution

#define SB_PITCH          	0x0001				// supports pitch control 
#define SB_PLAYBACKRATE   	0x0002				// supports playback rate control 
#define SB_VOLUME         	0x0004				// supports volume control 
#define SB_LRVOLUME       	0x0008				// separate left-right volume control
#define SB_SYNC           	0x0010				// synchronous driver will block while playing a buffer
#define SB_SAMPLEACCURATE 	0x0020				// returns sample-accurate position information
#define SB_DIRECTSOUND    	0x0040

#ifndef _INC_MMSYSTEM
#define SND_SYNC            0x0000  /* play synchronously (default) */
#define SND_ASYNC           0x0001  /* play asynchronously */
#define SND_NODEFAULT       0x0002  /* silence (!default) if sound not found */
#define SND_MEMORY          0x0004  /* pszSound points to a memory file */
#define SND_LOOP            0x0008  /* loop the sound until next sndPlaySound */
#define SND_NOSTOP          0x0010  /* don't stop any currently playing sound */
#endif

#define waveSuccess			0

#ifndef FALSE
#define FALSE               0
#endif

#ifndef TRUE
#define TRUE                1
#endif

#ifndef NULL
#ifdef __cplusplus
#define NULL    0
#else
#define NULL    ((void *)0)
#endif
#endif

#ifndef _WINDEF_
typedef unsigned char       BYTE;
typedef unsigned short      WORD;
typedef unsigned long       DWORD;
typedef float               FLOAT;
typedef int                 BOOL;
typedef BOOL                *PBOOL;
typedef int                 *PINT;
typedef BYTE                *PBYTE;
typedef WORD                *PWORD;
typedef DWORD               *PDWORD;
typedef FLOAT               *PFLOAT;
#endif

/*********************** Error Codes ************************************/

enum WaveErrors
{
	invalidValueErr              = -15000,		// Invalid parameter
	badWaveFormatErr             = -15001,		// Attempted to open with an unsupported waveform-audio format
	badDeviceIdErr               = -15002,		// Specified device identifier is out of range
	resourceAllocatedErr         = -15003,		// Specified resource is already allocated
	noDriverErr                  = -15004,		// No device driver is present
	noMemoryErr                  = -15005,		// Unable to allocate or lock memory
	invalidHandlerErr            = -15006,		// Specified device handle is invalid
	waveStillPlayingErr          = -15007,		// There are still buffers in the queue
	alreadyWaveInOpenErr         = -15008,		// Wave Input Device already open
	alreadyWaveOutOpenErr        = -15009,		// Wave Output Device already open
	noWaveInOpenErr              = -15010,		// Wave Input Device not opened
	noWaveOutOpenErr             = -15011,		// Wave Output Device not opened
	bufferUnpreparedErr          = -15012,		// The Wave buffer hasn't been prepared
	getWaveInDataTimeoutErr      = -15013,		// Wave Input Data timed out
	badTaskIDErr                 = -15014,		// Bad TaskID in async operations
	noDataRecordedErr            = -15015,		// No Input Data recorded on input buffer
	soundNotFoundErr             = -15016,		// Specified sound cannot be found
	noWaveDataErr                = -15017,		// No data in wave buffer
	waveFileCreatingErr          = -15018,		// Wave file creating error
	waveChunkCreationErr         = -15019,		// Chunk creation failure
	waveWritingErr               = -15020,		// Wave file writing error
	waveFileSeekErr              = -15021,		// Seek error in wave file
	waveFileCloseErr             = -15022,		// Wave file closing error
	waveFileOpenErr              = -15023,		// Wave file opening error (no file)
	waveReadingErr               = -15024,		// Wave file reading error
	noPCMWaveFileErr             = -15025,		// Wave file does not use a valid PCM format
	noValidDataBufferErr         = -15026,		// Data buffer invalid or unallocated
	noDataToPlayErr              = -15027,		// No Output Data to play on output buffer
	functionNotSupportedErr      = -15028,		// Function is not supported
	pitchNotSupportedErr         = -15029,		// Wave Output Device does not support pitch change
	playRateNotSupportedErr      = -15030,		// Wave Output Device does not support play rate change
	volumeNotSupportedErr        = -15031,		// Wave Output Device does not support volume change
	
	unknownErr                   = -15999		// God only knows !
};

/*********************** Function Prototypes ****************************/

short wiRecordWaveform      (int channels, int samples, double sampleRate,
                             int resolution, double *actualRate, int *samplesRecorded,
                             double leftBuffer[], double rightBuffer[]);
short wiStartRecord         (int channels, int samples, double sampleRate, int resolution,
                             double *actualRate, unsigned long *taskID);
short wiCheckRecord         (unsigned long taskID, int *samplesRecorded, short *doneFlag);
short wiReadRecord          (unsigned long taskID, int channels, int resolution, int samples,
                             double leftBuffer[], double rightBuffer[]);
short wiStopRecord          (unsigned long taskID);
short woGenerateWaveform    (int channels, int samples, double updateRate,
                             int resolution, int iterations, double *actualRate,
                             double leftBuffer[], double rightBuffer[],
                             unsigned long *taskID);
short woStopWaveform        (int channels, unsigned long taskID);
short mmPlaySound           (char fileName[], short asyncFlag);
short mmPlaySoundEx         (char fileName[], short asyncFlag, short loopFlag,
                             short playDefault, short stopSound);
void  mmStopSound           (void);
short wiInitBlaster         (void);
short wiInfo                (char productName[], unsigned short *driverVersion,
                             unsigned long *formatFlags, int *channels);
short wiOpen                (int channels, int sampleRate, int resolution);
short wiClose               (void);
short wiStart               (int samples, unsigned long *taskID);
short wiRestart             (unsigned long taskID, int samples);
short wiCheck               (unsigned long taskID, int *samplesRecorded, short *doneFlag);
short wiStop                (unsigned long taskID, int *samplesRecorded);
short wiClear               (unsigned long taskID);
short wiRecord              (int samples, int *samplesRecorded, short leftBuffer[],
                             short rightBuffer[]);
short wiToBuffers           (char waveBuffer[], short leftBuffer[], short rightBuffer[],
                             int samples, int channels, int resolution);
short wiXfer                (unsigned long taskID, int *samplesRecorded, short leftBuffer[],
                             short rightBuffer[]);
short wiScale               (int samples, int resolution, double gainAdjust, double offset,
                             short binaryArray[], double realArray[]);
short woInitBlaster         (void);
short woInfo                (char productName[], unsigned short *driverVersion,
                             unsigned long *formatFlags, int *channels,
                             unsigned long *supportFlags);
short woOpen                (int channels, int sampleRate, int resolution);
short woClose               (void);
short woStart               (int samples, int loops, short leftBuffer[], short rightBuffer[],
                             unsigned long *taskID);
short woCheck               (unsigned long taskID, int *samplesPlayed, short *doneFlag);
short woStop                (unsigned long taskID, int *samplesPlayed);
short woRestart             (unsigned long taskID);
short woClear               (unsigned long taskID);
short woPlay                (int samples, int loops, short leftBuffer[], short rightBuffer[]);
short woFromBuffers         (short leftBuffer[], short rightBuffer[], char waveBuffer[],
                             int samples, int channels, int resolution);
short woXfer                (unsigned long taskID, int samples, short leftBuffer[],
                             short rightBuffer[]);
short woScale               (int samples, int resolution, double realArray[], short binaryArray[]);
short woSetPitch            (double pitch);
short woSetPlayRate         (double playbackRate);
short woSetVolume           (double leftVolume, double rightVolume);
short woGetVolume           (double *leftVolume, double *rightVolume);
short wfSaveBuffer          (char fileName[], int channels, int sampleRate, int resolution);
short wfLoadBuffer          (char fileName[], int *channels, int *sampleRate,
                             int *resolution, int *samples);

#endif

