#include <ansi_c.h>
#include "Sbdaq.h"
#include <analysis.h>
#include <userint.h>
#include "general.h"
#include "globals.h"
#include "globdata.h"
#include "analizor.h"


extern int lab;

extern int analizor;
extern int panelAnalizorIsHided;
extern int achizitieEnable;

extern double bufferAchizCanal1[ESANTIOANE];
extern double bufferAchizCanal2[ESANTIOANE];
double bufferAchizSpectruCanal1[ESANTIOANE];
double bufferAchizSpectruCanal2[ESANTIOANE];

extern unsigned long IDTaskAchizitie;
extern double frecventaAchizCanal1;
extern double frecventaAchizCanal2;
	   
int handleGraphAnalizorCanal1 = -1;
int handleGraphAnalizorCanal2 = -1;

int afisCanal1;
int afisCanal2;

int linLogCanal1;
int linLogCanal2;

int CVICALLBACK OnOffAnalizor (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			wiStopRecord (IDTaskAchizitie);
			wiClose ();
			
			SetCtrlAttribute (lab, LAB_SPEC, ATTR_DIMMED, 0);
			HidePanel (analizor);
			panelAnalizorIsHided = true;

			break;
	}
	return 0;
}

int CVICALLBACK CallTimerAchizAnalizor (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int esantioaneInregistrate;                          
	
	switch (event) {
		case EVENT_TIMER_TICK:
			if (!achizitieEnable)
				{
				wiOpen (SB_STEREO, SB_RATE_44KHZ, SB_8BIT);

				wiRecord (ESANTIOANE, &esantioaneInregistrate,
								bufferAchizBinCanal1, bufferAchizBinCanal2);

				wiScale (ESANTIOANE, SB_8BIT, 1.0, 0.0,
								bufferAchizBinCanal1, bufferAchizCanal1);
				wiScale (ESANTIOANE, SB_8BIT, 1.0, 0.0,
								bufferAchizBinCanal2, bufferAchizCanal2);
				}
			break;
	}
	return 0;
}

int CVICALLBACK CallTimerAfisAnalizor (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{

	double curCanal1X, curCanal2X, curCanal1Y, curCanal2Y;
	
	switch (event) {
		case EVENT_TIMER_TICK:
		
			GetCtrlVal (analizor, ANALIZOR_ASCECRAN1, &afisCanal1);
			GetCtrlVal (analizor, ANALIZOR_ASCECRAN2, &afisCanal2);						

			Copy1D (bufferAchizCanal1, ESANTIOANE, bufferAchizSpectruCanal1);
			Copy1D (bufferAchizCanal2, ESANTIOANE, bufferAchizSpectruCanal2);						
			
			HanWin (bufferAchizSpectruCanal1, ESANTIOANE);						
			HanWin (bufferAchizSpectruCanal2, ESANTIOANE);
			
			if (!linLogCanal1)
				{
				SetCtrlAttribute (analizor, ANALIZOR_GRAPHCANAL1, ATTR_YMAP_MODE,
							  VAL_LOG);
				}
			else
				{
				SetCtrlAttribute (analizor, ANALIZOR_GRAPHCANAL1, ATTR_YMAP_MODE,
							  VAL_LINEAR);
				}

			if (!linLogCanal2)
				{
				SetCtrlAttribute (analizor, ANALIZOR_GRAPHCANAL2, ATTR_YMAP_MODE,
							  VAL_LOG);
				}
			else									
				{
				SetCtrlAttribute (analizor, ANALIZOR_GRAPHCANAL2, ATTR_YMAP_MODE,
							  VAL_LINEAR);
				}

			if (!afisCanal1)
				{
				Spectrum ( bufferAchizSpectruCanal1, ESANTIOANE );
				DeleteGraphPlot (analizor, ANALIZOR_GRAPHCANAL1, handleGraphAnalizorCanal1, VAL_IMMEDIATE_DRAW);
				handleGraphAnalizorCanal1 = PlotY (analizor, ANALIZOR_GRAPHCANAL1, bufferAchizSpectruCanal1,
										  ESANTIOANE_SPECTRU, VAL_DOUBLE, VAL_THIN_LINE,
										  VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_YELLOW);
				}

			GetGraphCursor (analizor, ANALIZOR_GRAPHCANAL1, 1, &curCanal1X, &curCanal1Y);

			SetCtrlVal (analizor, ANALIZOR_CISTIGCANAL1, curCanal1Y);
			SetCtrlVal (analizor, ANALIZOR_FRECVENTACANAL1, (double)curCanal1X*FRECVENTA_ESANTIONARE/ESANTIOANE);

			if (!afisCanal2)
				{
				Spectrum ( bufferAchizSpectruCanal2, ESANTIOANE );
				DeleteGraphPlot (analizor, ANALIZOR_GRAPHCANAL2, handleGraphAnalizorCanal2, VAL_IMMEDIATE_DRAW);
				handleGraphAnalizorCanal2 = PlotY (analizor, ANALIZOR_GRAPHCANAL2, bufferAchizSpectruCanal2,
										  ESANTIOANE_SPECTRU, VAL_DOUBLE, VAL_THIN_LINE,
										  VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_YELLOW);
				}

			GetGraphCursor (analizor, ANALIZOR_GRAPHCANAL2, 1, &curCanal2X, &curCanal2Y);									  

			SetCtrlVal (analizor, ANALIZOR_CISTIGCANAL2, curCanal2Y);
			SetCtrlVal (analizor, ANALIZOR_FRECVENTACANAL2, (double)curCanal2X*FRECVENTA_ESANTIONARE/ESANTIOANE);
									  
			break;
	}
	return 0;
}

int CVICALLBACK SetLinLog1 (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
				if (linLogCanal1)
					linLogCanal1 = false;
				else
					linLogCanal1 = true;				
			break;
	}
	return 0;
}

int CVICALLBACK SetLinLog2 (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
				if (linLogCanal2)
					linLogCanal2 = false;
				else
					linLogCanal2 = true;				

			break;
	}
	return 0;
}
