#include "Sbdaq.h"
#include <cvirte.h>		/* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include "globals.h"
#include "globdata.h"
#include "general.h"
#include "generator.h"
#include "osciloscop.h"
#include "analizor.h"
#include "ma.h"
#include "mf_mp.h"
#include "mia.h"



extern int generator;
extern int osciloscop;
extern int analizor;
extern int ma;
extern int mfmp;
extern int mia;
extern int panelGeneratorIsHided;
extern int panelOsciloscopIsHided;
extern int panelAnalizorIsHided;
extern int panelMAIsHided;
extern int panelMFMPIsHided;
extern int panelMIAIsHided;


extern unsigned long IDTaskAchizitie;
extern unsigned long IDTaskGenerare;


int main (int argc, char *argv[])
{
	if (InitCVIRTE (0, argv, 0) == 0)	/* Needed if linking in external compiler; harmless otherwise */
		return -1;	/* out of memory */
	if ((lab = LoadPanel (0, "general.uir", LAB)) < 0)
		return -1;
	DisplayPanel (lab);
	RunUserInterface ();
	return 0;
}

int CVICALLBACK CallGen (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			if ( panelGeneratorIsHided )
			{
				DisplayPanel( generator );
				panelGeneratorIsHided = false;
				panelMAIsHided = false;
				panelMFMPIsHided = false;
				panelMIAIsHided = false;
				SetCtrlAttribute (lab, LAB_GEN, ATTR_DIMMED, 1);
				SetCtrlAttribute (lab, LAB_MA, ATTR_DIMMED, 1);
				SetCtrlAttribute (lab, LAB_MFMP, ATTR_DIMMED, 1);
				SetCtrlAttribute (lab, LAB_MIA, ATTR_DIMMED, 1);
				break;
			}
			if ((generator = LoadPanel (0, "generator.uir", GENERATOR)) < 0)
				return -1;
			SetCtrlAttribute (lab, LAB_GEN, ATTR_DIMMED, 1);
			SetCtrlAttribute (lab, LAB_MA, ATTR_DIMMED, 1);
			SetCtrlAttribute (lab, LAB_MFMP, ATTR_DIMMED, 1);
			SetCtrlAttribute (lab, LAB_MIA, ATTR_DIMMED, 1);
			DisplayPanel (generator);
			RunUserInterface ();
			
			break;
	}
	return 0;
}

int CVICALLBACK CallSpec (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			if ( panelAnalizorIsHided )
			{
				DisplayPanel( analizor );
				panelAnalizorIsHided = false;
				SetCtrlAttribute (lab, LAB_SPEC, ATTR_DIMMED, 1);
				break;
			}
			if ((analizor = LoadPanel (0, "analizor.uir", ANALIZOR)) < 0)
				return -1;
			SetCtrlAttribute (lab, LAB_SPEC, ATTR_DIMMED, 1);
			DisplayPanel (analizor);
			RunUserInterface ();

			break;
	}
	return 0;
}

int CVICALLBACK CallOsc (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			if ( panelOsciloscopIsHided )
			{
				DisplayPanel( osciloscop );
				panelOsciloscopIsHided = false;
				SetCtrlAttribute (lab, LAB_OSC, ATTR_DIMMED, 1);				
				break;
			}
			if ((osciloscop = LoadPanel (0, "osciloscop.uir", OSCILOSCOP)) < 0)
				return -1;
			SetCtrlAttribute (lab, LAB_OSC, ATTR_DIMMED, 1);								
			DisplayPanel (osciloscop);
			RunUserInterface ();

			break;
	}
	return 0;
}


int CVICALLBACK CallMA (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			if ( panelMAIsHided )
			{
				DisplayPanel( ma );
				panelMAIsHided = false;
				panelMFMPIsHided = false;
				panelMIAIsHided = false;
				SetCtrlAttribute (lab, LAB_GEN, ATTR_DIMMED, 1);
				SetCtrlAttribute (lab, LAB_MA, ATTR_DIMMED, 1);
				SetCtrlAttribute (lab, LAB_MFMP, ATTR_DIMMED, 1);
				SetCtrlAttribute (lab, LAB_MIA, ATTR_DIMMED, 1);
				break;
			}
			if ((ma = LoadPanel (0, "MA.uir", MA)) < 0)
				return -1;
			SetCtrlAttribute (lab, LAB_GEN, ATTR_DIMMED, 1);
			SetCtrlAttribute (lab, LAB_MA, ATTR_DIMMED, 1);
			SetCtrlAttribute (lab, LAB_MFMP, ATTR_DIMMED, 1);
			SetCtrlAttribute (lab, LAB_MIA, ATTR_DIMMED, 1);
			DisplayPanel (ma);
			RunUserInterface ();
			
			break;
	}
	return 0;
}

int CVICALLBACK CallMFMP (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			if ( panelMFMPIsHided )
			{
				DisplayPanel( mfmp );
				panelMAIsHided = false;
				panelMFMPIsHided = false;
				panelMIAIsHided = false;
				SetCtrlAttribute (lab, LAB_GEN, ATTR_DIMMED, 1);
				SetCtrlAttribute (lab, LAB_MA, ATTR_DIMMED, 1);
				SetCtrlAttribute (lab, LAB_MFMP, ATTR_DIMMED, 1);
				SetCtrlAttribute (lab, LAB_MIA, ATTR_DIMMED, 1);
				break;
			}
			if ((mfmp = LoadPanel (0, "MF_MP.uir", MFMP)) < 0)
				return -1;
			SetCtrlAttribute (lab, LAB_GEN, ATTR_DIMMED, 1);
			SetCtrlAttribute (lab, LAB_MA, ATTR_DIMMED, 1);
			SetCtrlAttribute (lab, LAB_MFMP, ATTR_DIMMED, 1);
			SetCtrlAttribute (lab, LAB_MIA, ATTR_DIMMED, 1);
			DisplayPanel (mfmp);
			RunUserInterface ();
			
			break;
	}
	return 0;
}

int CVICALLBACK CallMIA (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			if ( panelMIAIsHided )
			{
				DisplayPanel( mia );
				panelMAIsHided = false;
				panelMFMPIsHided = false;
				panelMIAIsHided = false;
				SetCtrlAttribute (lab, LAB_GEN, ATTR_DIMMED, 1);
				SetCtrlAttribute (lab, LAB_MA, ATTR_DIMMED, 1);
				SetCtrlAttribute (lab, LAB_MFMP, ATTR_DIMMED, 1);
				SetCtrlAttribute (lab, LAB_MIA, ATTR_DIMMED, 1);
				break;
			}
			if ((mia = LoadPanel (0, "MIA.uir", MIA)) < 0)
				return -1;
			SetCtrlAttribute (lab, LAB_MIA, ATTR_DIMMED, 1);																
			DisplayPanel (mia);
			RunUserInterface ();
			
			break;
	}
	return 0;
}


int CVICALLBACK CallQUIT (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			woStopWaveform (SB_STEREO, IDTaskGenerare);
			woClose();
			wiStopRecord (IDTaskAchizitie);
			wiClose ();
			
			QuitUserInterface ( generator );		
			QuitUserInterface ( osciloscop );
			QuitUserInterface ( analizor );
			QuitUserInterface ( ma );
			QuitUserInterface ( mfmp );						
			QuitUserInterface ( mia );						
			QuitUserInterface (0);
			break;
	}
	return 0;
}

