#include <ansi_c.h>
#include <analysis.h>
#include "Sbdaq.h"
#include <cvirte.h>		/* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include "general.h"
#include "globals.h"
#include "globdata.h"
#include "generator.h"


extern int lab;

extern int generator;
extern unsigned long IDTaskGenerare;
extern int generationEnable = false;
extern int panelGeneratorIsHided = false;
extern int canal1GeneratorIsHided = false;
extern int canal2GeneratorIsHided = false;
extern double actualRate;
extern double updateRate = FRECVENTA_ESANTIONARE;

extern double bufferGenerareCanal1[ESANTIOANE];
extern double bufferGenerareCanal2[ESANTIOANE];

extern double amplitudineGenerareCanal1 = 1.0;
extern double amplitudineGenerareCanal2 = 1.0;
extern double fazaGenerareCanal1 = 0.0;
extern double fazaGenerareCanal2 = 0.0;
extern double frecventaGenerareCanal1 = 1012.06;
extern double frecventaGenerareCanal2 = 1012.06;


extern int tipulSemnalGeneratCanal1 = SINUS;
extern int tipulSemnalGeneratCanal2 = SINUS;
extern int numarIteratiiGenerare = -1;

int handleGraphCanal1 = -1;
int handleGraphCanal2 = -1;
int ascEcran1 = false;
int ascEcran2 = false;

int CVICALLBACK StartStopGen (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			if ( generationEnable  ) 
				{
				generationEnable = false;
				woStopWaveform (SB_STEREO, IDTaskGenerare);
				woClose();
				}
			else 
				generationEnable = true;				
			break;
	}
	return 0;
}

int CVICALLBACK SeteazaAmplitudineGenCanal1 (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (generator, GENERATOR_AMPLITUDINECANAL1,&amplitudineGenerareCanal1);
			break;
	}
	return 0;
}

int CVICALLBACK SeteazaAmplitudineGenCanal2 (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (generator, GENERATOR_AMPLITUDINECANAL2,&amplitudineGenerareCanal2);
			break;
	}
	return 0;
}


int CVICALLBACK OnOffGenerator (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int retCode;

	switch (event) {
		case EVENT_COMMIT:
		
			if ( generationEnable  ) 
				{
				generationEnable = false;
				woStopWaveform (SB_STEREO, IDTaskGenerare);
				woClose();
				}
			else 
				generationEnable = true;				

			retCode = woStopWaveform (SB_STEREO, IDTaskGenerare);					
			retCode = woClose();
			generationEnable = false;

			SetCtrlAttribute (lab, LAB_GEN, ATTR_DIMMED, 0);
			SetCtrlAttribute (lab, LAB_MA, ATTR_DIMMED, 0);
			SetCtrlAttribute (lab, LAB_MFMP, ATTR_DIMMED, 0);
			SetCtrlAttribute (lab, LAB_MIA, ATTR_DIMMED, 0);
			HidePanel (generator);
			panelGeneratorIsHided = true;
			break;
	}
	return 0;
}


void PregatireSemnalGenerare( int tip, double amplitudine, double frecventa, double faza,
								double buffer[ESANTIOANE])
{

	switch ( tip )
	{
		case SINUS:
			SineWave (ESANTIOANE, amplitudine, frecventa/((double)FRECVENTA_ESANTIONARE), 
							&faza, buffer);
			break;
		case TRIUNGHI:
			TriangleWave (ESANTIOANE, amplitudine,frecventa/((double)FRECVENTA_ESANTIONARE),
							&faza, buffer);
			break;
		case DREPTUNGHI:
			SquareWave (ESANTIOANE, amplitudine,frecventa/((double)FRECVENTA_ESANTIONARE),
							&faza, 50, buffer);
			break;
		case RANDOM:
			WhiteNoise (ESANTIOANE, amplitudine, 0, buffer);
			break;
		
		}

}

int CVICALLBACK CallTimeGen (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_TIMER_TICK:
			GetCtrlVal (generator, GENERATOR_TIPSEMNALCANAL1, &tipulSemnalGeneratCanal1);      
			GetCtrlVal (generator, GENERATOR_TIPSEMNALCANAL2, &tipulSemnalGeneratCanal2);      
			
			if ( generationEnable )
				{
				PregatireSemnalGenerare( tipulSemnalGeneratCanal1, amplitudineGenerareCanal1, 
							frecventaGenerareCanal1, fazaGenerareCanal1, bufferGenerareCanal1 );
				PregatireSemnalGenerare( tipulSemnalGeneratCanal2, amplitudineGenerareCanal2, 
							frecventaGenerareCanal2, fazaGenerareCanal2, bufferGenerareCanal2 );
				woGenerateWaveform (SB_STEREO, ESANTIOANE, updateRate, SB_8BIT,
							numarIteratiiGenerare, &actualRate,
							bufferGenerareCanal1, bufferGenerareCanal2,
							&IDTaskGenerare);
				}
			break;
	}
	return 0;
}

int CVICALLBACK CallTimerAfisGen (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_TIMER_TICK:
				
				GetCtrlVal (generator, GENERATOR_ASCECRAN1, &ascEcran1);
				GetCtrlVal (generator, GENERATOR_ASCECRAN2, &ascEcran2);						
				
				if ( generationEnable && !ascEcran1)
						{
						DeleteGraphPlot (generator, GENERATOR_GRAPHCANAL1, handleGraphCanal1, VAL_IMMEDIATE_DRAW);
						handleGraphCanal1 = PlotY (generator, GENERATOR_GRAPHCANAL1, bufferGenerareCanal1,
											  ESANTIOANE, VAL_DOUBLE, VAL_THIN_LINE,
											  VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
						}
				if ( generationEnable && !ascEcran2)
						{
						DeleteGraphPlot (generator, GENERATOR_GRAPHCANAL2, handleGraphCanal2, VAL_IMMEDIATE_DRAW);
						handleGraphCanal2 = PlotY (generator, GENERATOR_GRAPHCANAL2, bufferGenerareCanal2,
											  ESANTIOANE, VAL_DOUBLE, VAL_THIN_LINE,
											  VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
						}
		
			break;
	}
	return 0;
}

int CVICALLBACK SeteazaFrecventaGenCanal1 (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int nrPer;
	
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (generator, GENERATOR_FRECVENTACANAL1,&frecventaGenerareCanal1);	
			nrPer = ceil(frecventaGenerareCanal1*ESANTIOANE/FRECVENTA_ESANTIONARE);
			frecventaGenerareCanal1 = nrPer*FRECVENTA_ESANTIONARE/ESANTIOANE;
			SetCtrlVal (generator, GENERATOR_FRECVENTACANAL1,frecventaGenerareCanal1);			
			break;
	}
	return 0;
}

int CVICALLBACK SeteazaFrecventaGenCanal2 (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int nrPer;
	
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (generator, GENERATOR_FRECVENTACANAL2,&frecventaGenerareCanal2);	
			nrPer = ceil(frecventaGenerareCanal2*ESANTIOANE/FRECVENTA_ESANTIONARE);
			frecventaGenerareCanal2 = nrPer*FRECVENTA_ESANTIONARE/ESANTIOANE;
			SetCtrlVal (generator, GENERATOR_FRECVENTACANAL2,frecventaGenerareCanal2);			
			break;
	}
	return 0;
}
