#include <ansi_c.h>
#include "Sbdaq.h"
#include <analysis.h>
#include <userint.h>
#include "general.h"         
#include "globals.h"
#include "globdata.h"
#include "MA.h"

#define M0 1.0

extern int lab; 

extern int ma;
extern int panelMAIsHided = false;

extern unsigned long IDTaskGenerare;
extern int MAEnable = false;
extern double actualRate;
extern double updateRate;
extern int numarIteratiiGenerareMA = -1;

extern double purtatoare[ESANTIOANE];
extern double semnal_modulat[ESANTIOANE];
extern double modulatoare[ESANTIOANE];


static double frecventa_modulatoare = 107.67;
static double frecventa_purtatoare = 1033.59;
static double amplitudine_purtatoare = 0.5;
static int tip_modulatoare = SINUS;
static int nrPerPurtatoare;
static int nrPerModulatoare;
static double factor_umplere_modulatoare = 50.0;
static double m = 0.5; 
static int maps = false;


extern int handleGraphCanalMA1 = -1;
extern int handleGraphCanalMA2 = -1;
extern int ascMA1 = false;
extern int ascMA2 = false;
extern int msaup = false;						// se scoate la afisat modulatoarea


int CVICALLBACK StartStopGenMA (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			if ( MAEnable  ) 
				{
				MAEnable = false;
				woStopWaveform (SB_STEREO, IDTaskGenerare);
				woClose();
				}
			else 
				MAEnable = true;				
			break;
	}
	return 0;
}


void PregatireSemnalMAGenerare( int tip_modulatoare, double modulatoare[ESANTIOANE], 
		double semnal_modulat[ESANTIOANE], double factor_umplere_modulatoare )
{

	double faza = 0.0;
	double temp1[ESANTIOANE];
	double temp2[ESANTIOANE];

	GetCtrlVal (ma, MA_FRECVENTAMODMA,&frecventa_modulatoare);			
	nrPerModulatoare = ceil(frecventa_modulatoare*ESANTIOANE/FRECVENTA_ESANTIONARE);
	frecventa_modulatoare = nrPerModulatoare*FRECVENTA_ESANTIONARE/ESANTIOANE;
	SetCtrlVal (ma, MA_FRECVENTAMODMA,frecventa_modulatoare);			
		
	GetCtrlVal (ma, MA_FRECVENTAPURTATOAREMA,&frecventa_purtatoare);	
	nrPerPurtatoare = ceil(frecventa_purtatoare/frecventa_modulatoare)*nrPerModulatoare;
	frecventa_purtatoare = nrPerPurtatoare*FRECVENTA_ESANTIONARE/ESANTIOANE;
	SetCtrlVal (ma, MA_FRECVENTAPURTATOAREMA,frecventa_purtatoare);			

	
	switch ( tip_modulatoare )
	{
		case SINUS:
			
			faza = 0.0;			
						
			SineWave (ESANTIOANE, (double)M0, frecventa_modulatoare/((double)FRECVENTA_ESANTIONARE),
					  &faza, modulatoare);
			
			faza = 0.0;
			
			SineWave (ESANTIOANE, amplitudine_purtatoare, frecventa_purtatoare/((double)FRECVENTA_ESANTIONARE),
					  &faza, purtatoare);

			GetCtrlVal (ma, MA_MAPS, &maps);
			
			if ( !maps ){
				Set1D (temp1, ESANTIOANE, 1.0);
				Copy1D (modulatoare, ESANTIOANE, temp2);
				LinEv1D (modulatoare, ESANTIOANE, m, 0.0, temp2);
				Add1D (temp1, temp2, ESANTIOANE, temp1);
				Mul1D (temp1, purtatoare, ESANTIOANE, semnal_modulat);			
			}
			else
				Mul1D (modulatoare, purtatoare, ESANTIOANE, semnal_modulat);			
								  
			break;

		case TRIUNGHI:
		
			faza = 0.0;	
			
			TriangleWave (ESANTIOANE, M0, frecventa_modulatoare/((double)FRECVENTA_ESANTIONARE),
					 	 &faza, modulatoare);

			faza = 0.0;	
			
			SineWave (ESANTIOANE, amplitudine_purtatoare, frecventa_purtatoare/((double)FRECVENTA_ESANTIONARE),
					  &faza, purtatoare);

			GetCtrlVal (ma, MA_MAPS, &maps);
			
			if ( !maps ){
				Set1D (temp1, ESANTIOANE, 1.0);
				Copy1D (modulatoare, ESANTIOANE, temp2);
				LinEv1D (modulatoare, ESANTIOANE, m, 0.0, modulatoare);
				Add1D (temp1, modulatoare, ESANTIOANE, temp1);
				Mul1D (temp1, purtatoare, ESANTIOANE, semnal_modulat);			
			}
			else
				Mul1D (modulatoare, purtatoare, ESANTIOANE, semnal_modulat);			
								  
			break;
			
		case DREPTUNGHI:
			
			faza = 0.0;	
			
			SquareWave (ESANTIOANE, M0, frecventa_modulatoare/((double)FRECVENTA_ESANTIONARE),
						&faza, factor_umplere_modulatoare, modulatoare);

			faza = 0.0;				
						
			SineWave (ESANTIOANE, amplitudine_purtatoare, frecventa_purtatoare/((double)FRECVENTA_ESANTIONARE),
					  &faza, purtatoare);

			GetCtrlVal (ma, MA_MAPS, &maps);
			
			if ( !maps ){
				Set1D (temp1, ESANTIOANE, 1.0);
				Copy1D (modulatoare, ESANTIOANE, temp2);
				LinEv1D (modulatoare, ESANTIOANE, m, 0.0, temp2);
				Add1D (temp1, temp2, ESANTIOANE, temp1);
				Mul1D (temp1, purtatoare, ESANTIOANE, semnal_modulat);			
			}
			else
				Mul1D (modulatoare, purtatoare, ESANTIOANE, semnal_modulat);			
						
			break;

		
		}

}


int CVICALLBACK SeteazaFrecventaPurtatoareiMA (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{

	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (ma, MA_FRECVENTAPURTATOAREMA,&frecventa_purtatoare);	
			nrPerPurtatoare = ceil(frecventa_purtatoare/frecventa_modulatoare)*nrPerModulatoare;
			frecventa_purtatoare = nrPerPurtatoare*FRECVENTA_ESANTIONARE/ESANTIOANE;
			SetCtrlVal (ma, MA_FRECVENTAPURTATOAREMA,frecventa_purtatoare);			
			
			break;
	}
	return 0;
}

int CVICALLBACK SeteazaFrecventaModulatoareiMA (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{

	switch (event) {
		case EVENT_COMMIT:
			SetCtrlAttribute (ma, MA_FRECVENTAPURTATOAREMA, ATTR_DIMMED, 0);
			GetCtrlVal (ma, MA_FRECVENTAMODMA,&frecventa_modulatoare);			
			nrPerModulatoare = ceil(frecventa_modulatoare*ESANTIOANE/FRECVENTA_ESANTIONARE);
			frecventa_modulatoare = nrPerModulatoare*FRECVENTA_ESANTIONARE/ESANTIOANE;
			SetCtrlVal (ma, MA_FRECVENTAMODMA,frecventa_modulatoare);			

			break;
	}
	return 0;
}

int CVICALLBACK SeteazaFactorUmplereMA (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (ma, MA_FACTORUMPLEREMODMA,&factor_umplere_modulatoare);			
			
			break;
	}
	return 0;
}

int CVICALLBACK SeteazaIndiceModulatieMA (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (ma, MA_INDICEMODULATIEMA, &m);
			
			break;
	}
	return 0;
}

int CVICALLBACK SeteazaFormaModulatoareiMA (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (ma, MA_FORMAMODULATORMA, &tip_modulatoare);

			break;
	}
	return 0;
}

int CVICALLBACK SeteazaAmplPurtMA (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (ma, MA_AMPLPURTATOAREMA, &amplitudine_purtatoare);
			
			break;
	}
	return 0;
}



int CVICALLBACK CallTimeGenMA (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_TIMER_TICK:

			
			if ( MAEnable )
				{
				GetCtrlVal (ma, MA_MP, &msaup);
				PregatireSemnalMAGenerare( tip_modulatoare, modulatoare, semnal_modulat, factor_umplere_modulatoare);
				if ( !msaup )
					woGenerateWaveform (SB_STEREO, ESANTIOANE, updateRate, SB_8BIT,
								numarIteratiiGenerareMA, &actualRate,
								modulatoare, semnal_modulat,
								&IDTaskGenerare);
				else
					woGenerateWaveform (SB_STEREO, ESANTIOANE, updateRate, SB_8BIT,
								numarIteratiiGenerareMA, &actualRate,
								purtatoare, semnal_modulat,
								&IDTaskGenerare);
				}
			break;
	}
	return 0;
}

int CVICALLBACK CallTimerAfisGenMA (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_TIMER_TICK:
				
				GetCtrlVal (ma, MA_ASCECRAN1, &ascMA1);
				GetCtrlVal (ma, MA_ASCECRAN2, &ascMA2);						
				GetCtrlVal (ma, MA_MP, &msaup);						
				
				if ( MAEnable && !ascMA1)
						{
						if ( !msaup )
							{
							DeleteGraphPlot (ma, MA_GRAPHCANAL1, handleGraphCanalMA1, VAL_IMMEDIATE_DRAW);
							handleGraphCanalMA1 = PlotY (ma, MA_GRAPHCANAL1, modulatoare,
											  ESANTIOANE, VAL_DOUBLE, VAL_THIN_LINE,
											  VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
							}
						else
							{
							DeleteGraphPlot (ma, MA_GRAPHCANAL1, handleGraphCanalMA1, VAL_IMMEDIATE_DRAW);
							handleGraphCanalMA1 = PlotY (ma, MA_GRAPHCANAL1, purtatoare,
											  ESANTIOANE, VAL_DOUBLE, VAL_THIN_LINE,
											  VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
							}
						
						}

						
				if ( MAEnable && !ascMA2)
						{
						DeleteGraphPlot (ma, MA_GRAPHCANAL2, handleGraphCanalMA2, VAL_IMMEDIATE_DRAW);
						handleGraphCanalMA2 = PlotY (ma, MA_GRAPHCANAL2, semnal_modulat,
											  ESANTIOANE, VAL_DOUBLE, VAL_THIN_LINE,
											  VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
						}
		
			break;
	}
	return 0;
}


int CVICALLBACK OnOffMA (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int retCode;

	switch (event) {
		case EVENT_COMMIT:

			if ( MAEnable  ) 
				{
				MAEnable = false;
				woStopWaveform (SB_STEREO, IDTaskGenerare);
				woClose();
				}
			else 
				MAEnable = true;				
			
			
			retCode = woStopWaveform (SB_STEREO, IDTaskGenerare);					
			retCode = woClose();
			MAEnable = false;

			SetCtrlAttribute (lab, LAB_GEN, ATTR_DIMMED, 0);
			SetCtrlAttribute (lab, LAB_MA, ATTR_DIMMED, 0);
			SetCtrlAttribute (lab, LAB_MFMP, ATTR_DIMMED, 0);
			SetCtrlAttribute (lab, LAB_MIA, ATTR_DIMMED, 0);
			HidePanel (ma);
			panelMAIsHided = true;
			break;
	}
	return 0;
}

