#include <ansi_c.h>
#include "Sbdaq.h"
#include <analysis.h>
#include <userint.h>
#include "general.h"         
#include "globals.h"
#include "globdata.h"
#include "MF_MP.h"

#define pi 3.1415926

extern int lab; 

extern int mfmp;
extern int panelMFMPIsHided = false;

extern unsigned long IDTaskGenerare;
extern int MFMPEnable = false;

extern double actualRate;
extern double updateRate;
extern int numarIteratiiGenerareMFMP = -1;

extern double purtatoare[ESANTIOANE];
extern double semnal_modulat[ESANTIOANE];
extern double modulatoare[ESANTIOANE];

static double frecventa_modulatoare = 107.67;
static double frecventa_purtatoare = 1033.59;
static double amplitudine_purtatoare = 1.0;
static double amplitudine_modulatoare = 1.0;                
//static int tip_modulatoare = SINUS;
static int nrPerPurtatoare;
static int nrPerModulatoare = 6.0;
static double kfkphi = 100.0;
static double beta_alpha = 0.148; 
static int mp = false;


extern int handleGraphCanalMFMP1 = -1;
extern int handleGraphCanalMFMP2 = -1;
extern int ascMFMP1 = false;
extern int ascMFMP2 = false;


void PregatireSemnalMFMPGenerare( int tip_modulatoare, double modulatoare[ESANTIOANE], 
		double semnal_modulat[ESANTIOANE], double factor_umplere_modulatoare )
{

	double faza = 0.0;
	double purt[ESANTIOANE];
	double timp[ESANTIOANE];
	double modulatoare_copy[ESANTIOANE];
	double omega_purt;
	int i;

	GetCtrlVal (mfmp, MFMP_FRECVENTAMODMFMP,&frecventa_modulatoare);			
	nrPerModulatoare = ceil(frecventa_modulatoare*ESANTIOANE/FRECVENTA_ESANTIONARE);
	frecventa_modulatoare = nrPerModulatoare*FRECVENTA_ESANTIONARE/ESANTIOANE;
	SetCtrlVal (mfmp, MFMP_FRECVENTAMODMFMP,frecventa_modulatoare);			
		
	GetCtrlVal (mfmp, MFMP_FRECVENTAPURTATOAREMF,&frecventa_purtatoare);	
	nrPerPurtatoare = ceil(frecventa_purtatoare/frecventa_modulatoare)*nrPerModulatoare;
	frecventa_purtatoare = nrPerPurtatoare*FRECVENTA_ESANTIONARE/ESANTIOANE;
	SetCtrlVal (mfmp, MFMP_FRECVENTAPURTATOAREMF,frecventa_purtatoare);			

	beta_alpha = kfkphi*amplitudine_modulatoare;
	GetCtrlVal (mfmp, MFMP_MFMPCHECK, &mp);
	if (!mp)
		beta_alpha = beta_alpha/(2.0*pi*frecventa_modulatoare);
	SetCtrlVal (mfmp, MFMP_INDICEMODULATIEMFMP, beta_alpha);						
	
	Set1D (semnal_modulat, ESANTIOANE, 0.0);
	
	switch ( tip_modulatoare )
	{
		case SINUS:
			
			faza = 0.0;
			
			if (!mp)
				faza = 270.0;
			
			SineWave (ESANTIOANE, amplitudine_modulatoare, frecventa_modulatoare/((double)FRECVENTA_ESANTIONARE),
					  &faza, modulatoare);
				  
						
			Ramp (ESANTIOANE, 0.0, nrPerModulatoare/frecventa_modulatoare, timp);
			omega_purt = 2.0*pi*frecventa_purtatoare;
			LinEv1D (timp, ESANTIOANE, omega_purt, 0.0, purt);
			Copy1D (modulatoare, ESANTIOANE, modulatoare_copy);
			LinEv1D (modulatoare_copy, ESANTIOANE, beta_alpha, 0.0, modulatoare_copy);				
			Add1D (purt, modulatoare_copy, ESANTIOANE, semnal_modulat);
			for ( i = 0; i < ESANTIOANE; i++ )
				semnal_modulat[i] = amplitudine_purtatoare*cos(semnal_modulat[i]);
			break;

		case TRIUNGHI:

			break;
			
		case DREPTUNGHI:
						
			break;

		
		}

}


int CVICALLBACK OnOffMFMP (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int retCode;

	switch (event) {
		case EVENT_COMMIT:
			retCode = woStopWaveform (SB_STEREO, IDTaskGenerare);					
			retCode = woClose();
			MFMPEnable = false;

			SetCtrlAttribute (lab, LAB_GEN, ATTR_DIMMED, 0);
			SetCtrlAttribute (lab, LAB_MA, ATTR_DIMMED, 0);
			SetCtrlAttribute (lab, LAB_MFMP, ATTR_DIMMED, 0);
			SetCtrlAttribute (lab, LAB_MIA, ATTR_DIMMED, 0);       
			HidePanel (mfmp);
			panelMFMPIsHided = true;
			break;
	}
	return 0;
}


int CVICALLBACK SeteazaFrecventaPurtatoareiMFMP (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (mfmp, MFMP_FRECVENTAPURTATOAREMF,&frecventa_purtatoare);	
			nrPerPurtatoare = ceil(frecventa_purtatoare/frecventa_modulatoare)*nrPerModulatoare;
			frecventa_purtatoare = nrPerPurtatoare*FRECVENTA_ESANTIONARE/ESANTIOANE;
			SetCtrlVal (mfmp, MFMP_FRECVENTAPURTATOAREMF,frecventa_purtatoare);			

			break;
	}
	return 0;
}

int CVICALLBACK SeteazaFrecventaModulatoareiMFMP (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			SetCtrlAttribute (mfmp, MFMP_FRECVENTAPURTATOAREMF, ATTR_DIMMED, 0);
			GetCtrlVal (mfmp, MFMP_FRECVENTAMODMFMP,&frecventa_modulatoare);			
			nrPerModulatoare = ceil(frecventa_modulatoare*ESANTIOANE/FRECVENTA_ESANTIONARE);
			frecventa_modulatoare = nrPerModulatoare*FRECVENTA_ESANTIONARE/ESANTIOANE;
			SetCtrlVal (mfmp, MFMP_FRECVENTAMODMFMP,frecventa_modulatoare);			

			beta_alpha = kfkphi*amplitudine_modulatoare;
			GetCtrlVal (mfmp, MFMP_MFMPCHECK, &mp);
			if (!mp)
				beta_alpha = beta_alpha/(2.0*pi*frecventa_modulatoare);
			SetCtrlVal (mfmp, MFMP_INDICEMODULATIEMFMP, beta_alpha);						

			break;
	}
	return 0;
}


int CVICALLBACK SeteazaAmplPurtMFMP (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (mfmp, MFMP_AMPLPURTATOAREMFMP, &amplitudine_purtatoare);
			break;
	}
	return 0;
}

int CVICALLBACK SeteazaAmplModMFMP (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (mfmp, MFMP_AMPLMODMFMP, &amplitudine_modulatoare);
			beta_alpha = kfkphi*amplitudine_modulatoare;
			GetCtrlVal (mfmp, MFMP_MFMPCHECK, &mp);
			if (!mp)
				beta_alpha = beta_alpha/(2.0*pi*frecventa_modulatoare);
			SetCtrlVal (mfmp, MFMP_INDICEMODULATIEMFMP, beta_alpha);						

			break;
	}
	return 0;
}

int CVICALLBACK SeteazaKFKPHI (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (mfmp, MFMP_KFKPHI, &kfkphi);
			beta_alpha = kfkphi*amplitudine_modulatoare;
			GetCtrlVal (mfmp, MFMP_MFMPCHECK, &mp);
			if (!mp)
				beta_alpha = beta_alpha/(2.0*pi*frecventa_modulatoare);
			SetCtrlVal (mfmp, MFMP_INDICEMODULATIEMFMP, beta_alpha);						
			
			break;
	}
	return 0;
}


int CVICALLBACK CallTimerAfisGenMFMP (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_TIMER_TICK:
				GetCtrlVal (mfmp, MFMP_ASCECRAN1MFMP, &ascMFMP1);
				GetCtrlVal (mfmp, MFMP_ASCECRAN2MFMP, &ascMFMP2);						
				
				if ( MFMPEnable && !ascMFMP1)
						{
						DeleteGraphPlot (mfmp, MFMP_GRAPHCANAL1MFMP, handleGraphCanalMFMP1, VAL_IMMEDIATE_DRAW);
						handleGraphCanalMFMP1 = PlotY (mfmp, MFMP_GRAPHCANAL1MFMP, modulatoare,
											  ESANTIOANE, VAL_DOUBLE, VAL_THIN_LINE,
											  VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
						}
				if ( MFMPEnable && !ascMFMP2)
						{
						DeleteGraphPlot (mfmp, MFMP_GRAPHCANAL2MFMP, handleGraphCanalMFMP2, VAL_IMMEDIATE_DRAW);
						handleGraphCanalMFMP2 = PlotY (mfmp, MFMP_GRAPHCANAL2MFMP, semnal_modulat,
											  ESANTIOANE, VAL_DOUBLE, VAL_THIN_LINE,
											  VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
						}

			break;
	}
	return 0;
}

int CVICALLBACK CallTimeGenMFMP (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_TIMER_TICK:
			if ( MFMPEnable )
				{
				PregatireSemnalMFMPGenerare( SINUS, modulatoare, semnal_modulat, 50);
				woGenerateWaveform (SB_STEREO, ESANTIOANE, updateRate, SB_8BIT,
							numarIteratiiGenerareMFMP, &actualRate,
							modulatoare, semnal_modulat,
							&IDTaskGenerare);
				}

			break;
	}
	return 0;
}

int CVICALLBACK StartStopGenMFMP (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
		if ( MFMPEnable  ) 
				{
				MFMPEnable = false;
				woStopWaveform (SB_STEREO, IDTaskGenerare);
				woClose();
				}
			else 
				MFMPEnable = true;				
	
			break;
	}
	return 0;
}

