#include <analysis.h>
#include "Sbdaq.h"
#include <userint.h>
#include "general.h"
#include "globals.h"
#include "globdata.h"
#include "osciloscop.h"


extern int lab;

extern int osciloscop; 
extern int panelOsciloscopIsHided;
extern int achizitieEnable = false;

extern double bufferAchizCanal1[ESANTIOANE];
extern double bufferAchizCanal2[ESANTIOANE];
extern short bufferAchizBinCanal1[ESANTIOANE];
extern short bufferAchizBinCanal2[ESANTIOANE];
double bufferIntreTesut[ESANTIOANE];

extern double frecventaAchizCanal1;
extern double frecventaAchizCanal2;


static int handleGraphOscCanal1 = -1;
static int handleGraphOscCanal2 = -1;
static int handleGraphOscLissajoux = -1;
extern unsigned long IDTaskAchizitie;

static int ascEcranOsc = false;
static int modXY = false;

static double sensibilitateXCanal1 = 1.0;
static double sensibilitateXCanal2 = 1.0;
static int nrPerCanal1 = 0;
static int nrPerCanal2 = 0;



void CalculNumarPerioadeAchizitionate(double bufferCanal1[ESANTIOANE], 
		double bufferCanal2[ESANTIOANE], int *perCanal1, int *perCanal2)
{
	int i;
	double ac1, dc1, ac2, dc2;
	
	ACDCEstimator (bufferCanal1, ESANTIOANE, &ac1, &dc1);
	ACDCEstimator (bufferCanal2, ESANTIOANE, &ac2, &dc2);	
	
	*perCanal1 = 0;
	*perCanal2 = 0;
	
	for ( i = 0; i < ESANTIOANE - 1; i++ )
	{
		if ((bufferCanal1[i] - dc1)*(bufferCanal1[i+1] - dc1) < 0)
			(*perCanal1)++;
		if ((bufferCanal2[i] - dc2)*(bufferCanal2[i+1] - dc2) < 0)
			(*perCanal2)++;
	}
			
	*perCanal1 = (*perCanal1)/2 + 1;
	*perCanal2 = (*perCanal2)/2	+ 1;
}



int CVICALLBACK OnOffOsciloscop (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			wiStopRecord (IDTaskAchizitie);
			wiClose ();
			
			SetCtrlAttribute (lab, LAB_OSC, ATTR_DIMMED, 0);
			HidePanel (osciloscop);
			panelOsciloscopIsHided = true;

			break;
	}
	return 0;
}

int CVICALLBACK CallTimerAfisOsc (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{

	switch (event) {
		case EVENT_TIMER_TICK:
			GetCtrlVal (osciloscop, OSCILOSCOP_ASCECRAN, &ascEcranOsc);
			GetCtrlVal (osciloscop, OSCILOSCOP_MODXY, &modXY);
			
			CalculNumarPerioadeAchizitionate(bufferAchizCanal1, bufferAchizCanal2, 
								&nrPerCanal1, &nrPerCanal2);
			
			frecventaAchizCanal1 = (double)nrPerCanal1*FRECVENTA_ESANTIONARE/ESANTIOANE;
			frecventaAchizCanal2 = (double)nrPerCanal2*FRECVENTA_ESANTIONARE/ESANTIOANE;
			
			if (!ascEcranOsc && !modXY)
				{
				handleGraphOscCanal1 = -1;
				handleGraphOscCanal2 = -1;
				
				GetCtrlVal (osciloscop, OSCILOSCOP_SENSIBCANAL1X,&sensibilitateXCanal1);
				GetCtrlVal (osciloscop, OSCILOSCOP_SENSIBCANAL2X,&sensibilitateXCanal2);

				
				DeleteGraphPlot (osciloscop, OSCILOSCOP_GRAPH, handleGraphOscCanal1, VAL_IMMEDIATE_DRAW);
				DeleteGraphPlot (osciloscop, OSCILOSCOP_GRAPH, handleGraphOscCanal2, VAL_IMMEDIATE_DRAW);
				
				handleGraphOscCanal1 = PlotY (osciloscop, OSCILOSCOP_GRAPH, bufferAchizCanal1,
						  ESANTIOANE/sensibilitateXCanal1, VAL_DOUBLE, VAL_THIN_LINE,
						  VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_GREEN);
				
				SetCtrlVal (osciloscop, OSCILOSCOP_FRECVENTAOSCCANAL1, frecventaAchizCanal1);
				SetCtrlVal (osciloscop, OSCILOSCOP_SENSIBILITATECANAL1X, ESANTIOANE*1000/(NUMAR_DIVIZIUNI_OSC_X*sensibilitateXCanal1*FRECVENTA_ESANTIONARE));
				

				handleGraphOscCanal2 = PlotY (osciloscop, OSCILOSCOP_GRAPH, bufferAchizCanal2,
						  ESANTIOANE/sensibilitateXCanal2, VAL_DOUBLE, VAL_THIN_LINE,
						  VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_YELLOW);
						  
				SetCtrlVal (osciloscop, OSCILOSCOP_FRECVENTAOSCCANAL2, frecventaAchizCanal2);						  
				SetCtrlVal (osciloscop, OSCILOSCOP_SENSIBILITATECANAL2X, ESANTIOANE*1000/(NUMAR_DIVIZIUNI_OSC_X*sensibilitateXCanal2*FRECVENTA_ESANTIONARE));
				}
			
			if (!ascEcranOsc && modXY)
				{
				handleGraphOscLissajoux = -1;				
				DeleteGraphPlot (osciloscop, OSCILOSCOP_GRAPH, handleGraphOscLissajoux, VAL_IMMEDIATE_DRAW);
				handleGraphOscLissajoux = PlotXY (osciloscop,
							  OSCILOSCOP_GRAPH,bufferAchizCanal1,bufferAchizCanal2, 
							  ESANTIOANE,VAL_DOUBLE, VAL_DOUBLE,VAL_THIN_LINE, VAL_EMPTY_SQUARE,
							  VAL_SOLID, 1, VAL_GREEN);
				}
			
			break;
	}
	return 0;
}

int CVICALLBACK CallTimerAchizOsc (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{

	int retCode, esantioaneInregistrate;
	
	switch (event) {
		case EVENT_TIMER_TICK:
				
				retCode = wiOpen (SB_STEREO, SB_RATE_44KHZ, SB_8BIT);

				retCode = wiRecord (ESANTIOANE, &esantioaneInregistrate,
								bufferAchizBinCanal1, bufferAchizBinCanal2);

				retCode = wiScale (ESANTIOANE, SB_8BIT, 1.0, 0.0,
								bufferAchizBinCanal1, bufferAchizCanal1);
				retCode = wiScale (ESANTIOANE, SB_8BIT, 1.0, 0.0,
								bufferAchizBinCanal2, bufferAchizCanal2);
								

				achizitieEnable = true;				

			break;
	}
	return 0;
}

